@@ -15,7 +15,7 @@
  * The Original Code is mozila.org code.
  *
  * The Initial Developer of the Original Code is
- * Mozilla Corporation
+ * Mozilla Foundation
  * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
@@ -71,13 +71,13 @@
 #include "nsJSEnvironment.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMClassInfo.h"
-#include "nsIDOMFileInternal.h"
 #include "nsCExternalHandlerService.h"
 #include "nsIStreamConverterService.h"
 #include "nsEventDispatcher.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsLayoutStatics.h"
 #include "nsIScriptObjectPrincipal.h"
+#include "nsFileDataProtocolHandler.h"
 
 #define LOAD_STR "load"
 #define ERROR_STR "error"
@@ -88,6 +88,7 @@
 #define LOADEND_STR "loadend"
 
 #define NS_PROGRESS_EVENT_INTERVAL 50
+const PRUint64 kUnknownSize = PRUint64(-1);
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsDOMFileReader)
 
@@ -109,6 +110,8 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(nsDOMFileReader,
   NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mChannel)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
+DOMCI_DATA(FileReader, nsDOMFileReader)
+
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(nsDOMFileReader)
   NS_INTERFACE_MAP_ENTRY(nsIDOMFileReader)
   NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
@@ -117,7 +120,7 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(nsDOMFileReader)
   NS_INTERFACE_MAP_ENTRY(nsIJSNativeInitializer)
   NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
   NS_INTERFACE_MAP_ENTRY(nsICharsetDetectionObserver)
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(FileReader)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(FileReader)
 NS_INTERFACE_MAP_END_INHERITING(nsXHREventTarget)
 
 NS_IMPL_ADDREF_INHERITED(nsDOMFileReader, nsXHREventTarget)
@@ -141,7 +144,7 @@ nsDOMFileReader::SetOnloadend(nsIDOMEventListener* aOnloadend)
 NS_IMETHODIMP
 nsDOMFileReader::Notify(const char *aCharset, nsDetectionConfident aConf)
 {
-  CopyASCIItoUTF16(aCharset, mCharset);
+  mCharset = aCharset;
   return NS_OK;
 }
 
@@ -262,20 +265,20 @@ nsDOMFileReader::GetError(nsIDOMFileError** aError)
 }
 
 NS_IMETHODIMP
-nsDOMFileReader::ReadAsBinaryString(nsIDOMFile* aFile)
+nsDOMFileReader::ReadAsBinaryString(nsIDOMBlob* aFile)
 {
   return ReadFileContent(aFile, EmptyString(), FILE_AS_BINARY);
 }
 
 NS_IMETHODIMP
-nsDOMFileReader::ReadAsText(nsIDOMFile* aFile,
+nsDOMFileReader::ReadAsText(nsIDOMBlob* aFile,
                             const nsAString &aCharset)
 {
   return ReadFileContent(aFile, aCharset, FILE_AS_TEXT);
 }
 
 NS_IMETHODIMP
-nsDOMFileReader::ReadAsDataURL(nsIDOMFile* aFile)
+nsDOMFileReader::ReadAsDataURL(nsIDOMBlob* aFile)
 {
   return ReadFileContent(aFile, EmptyString(), FILE_AS_DATAURL);
 }
@@ -461,6 +464,8 @@ nsDOMFileReader::OnStopRequest(nsIRequest *aRequest,
       rv = GetAsDataURL(mFile, mFileData, mDataLen, mResult);
       break;
   }
+  
+  mResult.SetIsVoid(PR_FALSE);
 
   FreeFileData();
 
@@ -479,10 +484,11 @@ nsDOMFileReader::OnStopRequest(nsIRequest *aRequest,
 // Helper methods
 
 nsresult
-nsDOMFileReader::ReadFileContent(nsIDOMFile* aFile,
+nsDOMFileReader::ReadFileContent(nsIDOMBlob* aFile,
                                  const nsAString &aCharset,
                                  eDataFormat aDataFormat)
 {
+  nsresult rv;
   NS_ENSURE_TRUE(aFile, NS_ERROR_NULL_POINTER);
 
   //Implicit abort to clear any other activity going on
@@ -494,26 +500,28 @@ nsDOMFileReader::ReadFileContent(nsIDOMFile* aFile,
   mReadyState = nsIDOMFileReader::EMPTY;
   FreeFileData();
 
+  mFile = aFile;
   mDataFormat = aDataFormat;
-  mCharset = aCharset;
-
-  //Obtain the nsDOMFile's underlying nsIFile
-  nsresult rv;
-  nsCOMPtr<nsIDOMFileInternal> domFile(do_QueryInterface(aFile));
-  rv = domFile->GetInternalFile(getter_AddRefs(mFile));
-  NS_ENSURE_SUCCESS(rv, rv);
+  CopyUTF16toUTF8(aCharset, mCharset);
 
   //Establish a channel with our file
-  nsCOMPtr<nsIURI> uri;
-  rv = NS_NewFileURI(getter_AddRefs(uri), mFile);
-  NS_ENSURE_SUCCESS(rv, rv);
+  {
+    // Hold the internal URL alive only as long as necessary
+    // After the channel is created it will own whatever is backing
+    // the DOMFile.
+    nsDOMFileInternalUrlHolder urlHolder(mFile, mPrincipal);
 
-  rv = NS_NewChannel(getter_AddRefs(mChannel), uri);
-  NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr<nsIURI> uri;
+    rv = NS_NewURI(getter_AddRefs(uri), urlHolder.mUrl);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = NS_NewChannel(getter_AddRefs(mChannel), uri);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   //Obtain the total size of the file before reading
-  mReadTotal = -1;
-  mFile->GetFileSize(&mReadTotal);
+  mReadTotal = kUnknownSize;
+  mFile->GetSize(&mReadTotal);
 
   rv = mChannel->AsyncOpen(this, nsnull);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -568,14 +576,23 @@ nsDOMFileReader::DispatchProgressEvent(const nsAString& aType)
   if (!progress)
     return;
 
-  progress->InitProgressEvent(aType, PR_FALSE, PR_FALSE, mReadTotal >= 0,
-                              mReadTransferred, PR_MAX(mReadTotal, 0));
+  PRBool known;
+  PRUint64 size;
+  if (mReadTotal != kUnknownSize) {
+    known = PR_TRUE;
+    size = mReadTotal;
+  } else {
+    known = PR_FALSE;
+    size = 0;
+  }
+  progress->InitProgressEvent(aType, PR_FALSE, PR_FALSE, known,
+                              mReadTransferred, size);
 
   this->DispatchDOMEvent(nsnull, event, nsnull, nsnull);
 }
 
 nsresult
-nsDOMFileReader::GetAsText(const nsAString &aCharset,
+nsDOMFileReader::GetAsText(const nsACString &aCharset,
                            const char *aFileData,
                            PRUint32 aDataLen,
                            nsAString& aResult)
@@ -583,7 +600,7 @@ nsDOMFileReader::GetAsText(const nsAString &aCharset,
   nsresult rv;
   nsCAutoString charsetGuess;
   if (!aCharset.IsEmpty()) {
-    CopyUTF16toUTF8(aCharset, charsetGuess);
+    charsetGuess = aCharset;
   } else {
     rv = GuessCharset(aFileData, aDataLen, charsetGuess);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -602,7 +619,7 @@ nsDOMFileReader::GetAsText(const nsAString &aCharset,
 }
 
 nsresult
-nsDOMFileReader::GetAsDataURL(nsIFile *aFile,
+nsDOMFileReader::GetAsDataURL(nsIDOMBlob *aFile,
                               const char *aFileData,
                               PRUint32 aDataLen,
                               nsAString& aResult)
@@ -610,21 +627,17 @@ nsDOMFileReader::GetAsDataURL(nsIFile *aFile,
   aResult.AssignLiteral("data:");
 
   nsresult rv;
-  nsCOMPtr<nsIMIMEService> mimeService =
-    do_GetService(NS_MIMESERVICE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCAutoString contentType;
-  rv = mimeService->GetTypeFromFile(aFile, contentType);
-  if (NS_SUCCEEDED(rv)) {
-    AppendUTF8toUTF16(contentType, aResult);
+  nsString contentType;
+  rv = aFile->GetType(contentType);
+  if (NS_SUCCEEDED(rv) && !contentType.IsEmpty()) {
+    aResult.Append(contentType);
   } else {
     aResult.AppendLiteral("application/octet-stream");
   }
   aResult.AppendLiteral(";base64,");
 
   PRUint32 totalRead = 0;
-  do {
+  while (aDataLen > totalRead) {
     PRUint32 numEncode = 4096;
     PRUint32 amtRemaining = aDataLen - totalRead;
     if (numEncode > amtRemaining)
@@ -642,8 +655,7 @@ nsDOMFileReader::GetAsDataURL(nsIFile *aFile,
     PR_Free(base64);
 
     totalRead += numEncode;
-
-  } while (aDataLen > totalRead);
+  }
 
   return NS_OK;
 }
@@ -699,7 +711,9 @@ nsDOMFileReader::GuessCharset(const char *aFileData,
   }
 
   nsresult rv;
-  if (detector) {
+  // The charset detector doesn't work for empty (null) aFileData. Testing
+  // aDataLen instead of aFileData so that we catch potential errors.
+  if (detector && aDataLen != 0) {
     mCharset.Truncate();
     detector->Init(this);
 
@@ -711,7 +725,7 @@ nsDOMFileReader::GuessCharset(const char *aFileData,
     rv = detector->Done();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    CopyUTF16toUTF8(mCharset, aCharset);
+    aCharset = mCharset;
   } else {
     // no charset detector available, check the BOM
     unsigned char sniffBuf[4];
