@@ -37,19 +37,17 @@
  * ***** END LICENSE BLOCK ***** */
 #include "stdafx.h"
 
-#include "jni.h"
 #include "npapi.h"
 
+#include "jsapi.h"
+#include "jscntxt.h"
+
 #include "nsISupports.h"
 
 #ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
 #include "XPConnect.h"
 #endif
 
-#ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
-#include "LiveConnect.h"
-#endif
-
 #include "LegacyPlugin.h"
 
 #ifdef XPC_IDISPATCH_SUPPORT
@@ -63,8 +61,6 @@
 #include "nsIScriptContext.h"
 #include "nsIURI.h"
 #include "nsIJSContextStack.h"
-#include "jsapi.h"
-#include "jscntxt.h"
 #include "nsIScriptSecurityManager.h"
 #endif
 
@@ -244,26 +240,10 @@ NPError NPP_Initialize(void)
 void NPP_Shutdown(void)
 {
     ATLTRACE(_T("NPP_Shutdown()\n"));
-#ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
-    liveconnect_Shutdown();
-#endif
     _Module.Unlock();
 }
 
 
-// NPP_GetJavaClass
-//
-//    Return the Java class representing this plugin
-//
-jref NPP_GetJavaClass(void)
-{
-    ATLTRACE(_T("NPP_GetJavaClass()\n"));
-#ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
-    return liveconnect_GetJavaClass();
-#endif
-    return NULL;
-}
-
 #define MIME_OLEOBJECT1   "application/x-oleobject"
 #define MIME_OLEOBJECT2   "application/oleobject"
 
@@ -379,7 +359,7 @@ MozAxAutoPushJSContext::MozAxAutoPushJSContext(JSContext *cx,
     // See if there are any scripts on the stack.
     // If not, we need to add a dummy frame with a principal.
     PRBool hasScript = PR_FALSE;
-    JSStackFrame* tempFP = cx->fp;
+    JSStackFrame* tempFP = cx->fp();
     while (tempFP)
     {
         if (tempFP->script)
@@ -464,6 +444,8 @@ WillHandleCLSID(const CLSID &clsid, PluginInstanceData *pData)
     PRBool classExists = PR_FALSE;
     nsCOMPtr<nsIURI> uri;
     MozAxPlugin::GetCurrentLocation(pData->pPluginInstance, getter_AddRefs(uri));
+
+    JSAutoRequest req(cx);
     MozAxAutoPushJSContext autoContext(cx, uri);
     dispSupport->IsClassSafeToHost(cx, cid, PR_TRUE, &classExists, &isSafe);
     if (classExists && !isSafe)
@@ -578,7 +560,26 @@ CreateControl(const CLSID &clsid, PluginInstanceData *pData, PropertyList &pl, L
     PRBool hostSafeControlsOnly;
     PRBool downloadControlsIfMissing;
 #if defined(MOZ_ACTIVEX_PLUGIN_XPCONNECT) && defined(XPC_IDISPATCH_SUPPORT)
+#ifdef MOZ_FLASH_ACTIVEX_PATCH
+    GUID flashGUID;
+    ::CLSIDFromString(_T("{D27CDB6E-AE6D-11CF-96B8-444553540000}"), &flashGUID);
+
+    // HACK: Allow anything but downloading for the wrapped Flash control.
+    PRUint32 hostingFlags;
+    if (clsid == flashGUID)
+    {
+      hostingFlags = (nsIActiveXSecurityPolicy::HOSTING_FLAGS_HOST_SAFE_OBJECTS |
+                      nsIActiveXSecurityPolicy::HOSTING_FLAGS_SCRIPT_SAFE_OBJECTS |
+                      nsIActiveXSecurityPolicy::HOSTING_FLAGS_SCRIPT_ALL_OBJECTS |
+                      nsIActiveXSecurityPolicy::HOSTING_FLAGS_HOST_ALL_OBJECTS);
+    }
+    else
+    {
+      hostingFlags = MozAxPlugin::PrefGetHostingFlags();
+    }
+#else
     PRUint32 hostingFlags = MozAxPlugin::PrefGetHostingFlags();
+#endif
     if (hostingFlags & nsIActiveXSecurityPolicy::HOSTING_FLAGS_HOST_SAFE_OBJECTS &&
         !(hostingFlags & nsIActiveXSecurityPolicy::HOSTING_FLAGS_HOST_ALL_OBJECTS))
     {
@@ -807,7 +808,7 @@ NewControl(const char *pluginType,
             char szCLSID[kCLSIDLen];
             if (strlen(argv[i]) < sizeof(szCLSID))
             {
-                if (strnicmp(argv[i], "CLSID:", 6) == 0)
+                if (_strnicmp(argv[i], "CLSID:", 6) == 0)
                 {
                     _snprintf(szCLSID, kCLSIDLen - 1, "{%s}", argv[i]+6);
                 }
@@ -865,7 +866,7 @@ NewControl(const char *pluginType,
         else 
         {
             CComBSTR paramName;
-            if (strnicmp(argn[i], "PARAM_", 6) == 0)
+            if (_strnicmp(argn[i], "PARAM_", 6) == 0)
             {
                 paramName = argn[i] + 6;
             }
@@ -921,7 +922,7 @@ NewControl(const char *pluginType,
 //    create a new plugin instance 
 //    handle any instance specific code initialization here
 //
-NPError NP_LOADDS NPP_New(NPMIMEType pluginType,
+NPError NPP_New(NPMIMEType pluginType,
                 NPP instance,
                 uint16_t mode,
                 int16_t argc,
@@ -954,6 +955,8 @@ NPError NP_LOADDS NPP_New(NPMIMEType pluginType,
     MozAxPlugin::AddRef();
 #endif
 
+    CoInitializeEx(NULL, COINIT_MULTITHREADED);
+
     NPError rv = NPERR_GENERIC_ERROR;
     /* if (strcmp(pluginType, MIME_OLEOBJECT1) == 0 ||
            strcmp(pluginType, MIME_OLEOBJECT2) == 0) */
@@ -972,6 +975,7 @@ NPError NP_LOADDS NPP_New(NPMIMEType pluginType,
 #ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
         MozAxPlugin::Release();
 #endif
+        CoUninitialize();
         return rv;
     }
 
@@ -985,7 +989,7 @@ NPError NP_LOADDS NPP_New(NPMIMEType pluginType,
 //
 //    Deletes a plug-in instance and releases all of its resources.
 //
-NPError NP_LOADDS
+NPError
 NPP_Destroy(NPP instance, NPSavedData** save)
 {
     ATLTRACE(_T("NPP_Destroy()\n"));
@@ -1035,6 +1039,8 @@ NPP_Destroy(NPP instance, NPSavedData** save)
 
     instance->pdata = 0;
 
+    CoUninitialize();
+
     return NPERR_NO_ERROR;
 
 }
@@ -1056,7 +1062,7 @@ NPP_Destroy(NPP instance, NPSavedData** save)
 //  must be maintained by the plug-in to correctly handle the degenerate
 //  case.
 //
-NPError NP_LOADDS
+NPError
 NPP_SetWindow(NPP instance, NPWindow* window)
 {
     ATLTRACE(_T("NPP_SetWindow()\n"));
@@ -1123,7 +1129,7 @@ NPP_SetWindow(NPP instance, NPWindow* window)
 //  NPNormal  (e.g. *streamtype = NPNormal)...the NPE_StreamAsFile function will
 //  never be called in this case
 //
-NPError NP_LOADDS
+NPError
 NPP_NewStream(NPP instance,
               NPMIMEType type,
               NPStream *stream, 
@@ -1149,7 +1155,7 @@ NPP_NewStream(NPP instance,
 //    The stream is done transferring and here is a pointer to the file in the cache
 //    This function is only called if the streamtype was set to NPAsFile.
 //
-void NP_LOADDS
+void
 NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
 {
     ATLTRACE(_T("NPP_StreamAsFile()\n"));
@@ -1182,7 +1188,7 @@ int32_t STREAMBUFSIZE = 0X0FFFFFFF;   // we are reading from a file in NPAsFile
 //    The number of bytes that a plug-in is willing to accept in a subsequent
 //    NPO_Write call.
 //
-int32_t NP_LOADDS
+int32_t
 NPP_WriteReady(NPP instance, NPStream *stream)
 {
     return STREAMBUFSIZE;  
@@ -1193,7 +1199,7 @@ NPP_WriteReady(NPP instance, NPStream *stream)
 //
 //    Provides len bytes of data.
 //
-int32_t NP_LOADDS
+int32_t
 NPP_Write(NPP instance, NPStream *stream, int32_t offset, int32_t len, void *buffer)
 {   
     return len;
@@ -1207,7 +1213,7 @@ NPP_Write(NPP instance, NPStream *stream, int32_t offset, int32_t len, void *buf
 //    that it was complete, because there was some error, or because 
 //    the user aborted it.
 //
-NPError NP_LOADDS
+NPError
 NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 {
     // because I am handling the stream as a file, I don't do anything here...
@@ -1221,7 +1227,7 @@ NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 //
 //    Printing the plugin (to be continued...)
 //
-void NP_LOADDS
+void
 NPP_Print(NPP instance, NPPrint* printInfo)
 {
     if(printInfo == NULL)   // trap invalid parm
@@ -1266,7 +1272,7 @@ NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
     }
 }
 
-NPError	NP_LOADDS
+NPError
 NPP_GetValue(NPP instance, NPPVariable variable, void *value)
 {
     NPError rv = NPERR_GENERIC_ERROR;
@@ -1276,7 +1282,7 @@ NPP_GetValue(NPP instance, NPPVariable variable, void *value)
     return rv;
 }
 
-NPError	NP_LOADDS
+NPError
 NPP_SetValue(NPP instance, NPNVariable variable, void *value)
 {
     return NPERR_GENERIC_ERROR;
