@@ -35,7 +35,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '11.1.4-05.js';
 
 var summary = "11.1.4 - XML Initializer - Comment hiding parsing/scanning";
 var BUGNUMBER = 311157;
@@ -45,13 +44,12 @@ var expect;
 printBugNumber(BUGNUMBER);
 START(summary);
 
-XML.ignoreWhitespace = false;
-
-var x = <bye> <![CDATA[ duh ]]>
-    there </bye>;
+var x = <hi> <!-- duh -->
+    there </hi>;
 
 actual = x.toString();
-expect = '  duh \n    there ';
+expect = '\n    there ';
+
 TEST(1, expect, actual);
 
 END();
