@@ -20,7 +20,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,29 +36,22 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = 'regress-356238-03.js';
 
-var BUGNUMBER = 356238;
-var summary = 'bug 356238';
-var actual = 'No Error';
-var expect = 'No Error';
+var BUGNUMBER = 355474;
+var summary = 'Iterating over XML with WAY_TOO_MUCH_GC';
+var actual = '';
+var expect = '';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-var xml = <xml><child><a/></child></xml>;
-var child = xml.child[0];
-
-try {
-  child.insertChildBefore(null, xml.child);
-  actual = "insertChildBefore succeded when it should throw an exception";
-} catch (e) {
+expect = '<a>text</a>';
+for each (var i in <><a>text</a></>)
+{
+    printStatus(i.toXMLString());
+    actual = i.toXMLString();
 }
 
-var list = child.*;
-var text = uneval(list[1]);
-if (!/(undefined|\(void 0\))/.test(text))
-  throw "child got unexpected second element: "+text;
-
 TEST(1, expect, actual);
+
 END();
