@@ -36,32 +36,31 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = 'regress-450871-01.js';
 
-var summary = 'Do not crash: __proto__ = <x/>; <x/>.lastIndexOf(this, false)';
-var BUGNUMBER = 450871;
-var actual = 'No Crash';
-var expect = 'No Crash';
+var summary = 'Do not assert: pobj_ == obj2';
+var BUGNUMBER = 462734;
+var actual = '';
+var expect = '';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-if (typeof window == 'object')
+var save__proto__ = this.__proto__;
+
+try
 {
-    actual = expect = 'Test skipped for browser based tests due destruction of the prototype';
+    for (x in function(){}) (<x><y/></x>);
+    this.__defineGetter__("x", Function);
+    __proto__ = x;
+    prototype += <x><y/></x>;
 }
-else
+catch(ex)
 {
-    try
-    {
-        __proto__ = <x/>; 
-        <x/>.lastIndexOf(this, false);
-    }
-    catch(ex)
-    {
-    }
+    print(ex + '');
 }
 
+__proto__ = save__proto__;
+
 TEST(1, expect, actual);
 
 END();
