@@ -36,35 +36,30 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.7.4.2-4.js';
 
 /**
-   File Name:          15.7.4.2-4.js
-   ECMA Section:       15.7.4.2.1 Number.prototype.toString()
-   Description:
-   If the radix is the number 10 or not supplied, then this number value is
-   given as an argument to the ToString operator; the resulting string value
-   is returned.
+   File Name:          15.7.3.4-4.js
+   ECMA Section:       15.7.3.4 Number.NaN
+   Description:        All value properties of the Number object should have
+   the attributes [DontEnum, DontDelete, ReadOnly]
 
-   If the radix is supplied and is an integer from 2 to 36, but not 10, the
-   result is a string, the choice of which is implementation dependent.
-
-   The toString function is not generic; it generates a runtime error if its
-   this value is not a Number object. Therefore it cannot be transferred to
-   other kinds of objects for use as a method.
+   this test checks the DontEnum attribute of Number.NaN
 
    Author:             christine@netscape.com
    Date:               16 september 1997
 */
-var SECTION = "15.7.4.2-4";
+
+var SECTION = "15.7.3.4-4";
 var VERSION = "ECMA_1";
 startTest();
+var TITLE   = "Number.NaN";
 
-writeHeaderToLog( SECTION + " Number.prototype.toString()");
+writeHeaderToLog( SECTION + " " + TITLE);
 
-new TestCase(SECTION, 
-	     "o = 3; o.toString = Number.prototype.toString; o.toString()",            
-	     "3",   
-	     eval("o = 3; o.toString = Number.prototype.toString; o.toString()") );
+new TestCase( SECTION,
+	      "var string = ''; for ( prop in Number ) { string += ( prop == 'NaN' ) ? prop : '' } string;",
+	      "",
+	      eval("var string = ''; for ( prop in Number ) { string += ( prop == 'NaN' ) ? prop : '' } string;")
+  );
 
 test();
