@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Jesse Ruderman
@@ -35,12 +35,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-416737-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 416737;
-var summary = 'Do not assert: *pc == JSOP_GETARG';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 349482;
+var summary = 'Decompiling try/catch in for..in should not crash';
+var actual = 'No Crash';
+var expect = 'No Crash';
+
 
 //-----------------------------------------------------------------------------
 test();
@@ -51,11 +51,11 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  (function() { (function([]){ function n(){} })(1) });
+ 
+  var f = function() { for(p in {})   try{}catch(e){}   };
+  print(f.toString());
 
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
-
