@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jeff Walden
+ * Contributor(s): Brendan Eich
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-466905-04.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 466905;
-var summary = 'Prototypes of sandboxed arrays';
+var BUGNUMBER = 350312;
+var summary = 'Accessing wrong stack slot with nested catch/finally';
 var actual = '';
 var expect = '';
 
@@ -53,30 +52,93 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  if (typeof evalcx != 'function')
+  function createPrint(obj)
   {
-    expect = actual = 'Test skipped: requires evalcx support';
+    return new Function("actual += " + obj + " + ','; " +
+			"print(" + obj + ");");
   }
-  else
-  {
-    expect = true;
 
-    function createArray()
-    {
-      var a;                
-      for (var i = 0; i < 10; i++)
-        a = [1, 2, 3, 4, 5];
-      return a;
+  function createThrow(obj)
+  {
+    return new Function("throw " + obj + "; ");
+  }
+
+
+  function f(a, b, c)
+  {
+    try {
+      a();
+    } catch (e if e == null) {
+      b();
+    } finally {
+      c();
     }
-
-    var sandbox = evalcx("lazy");
-    sandbox.createArray = createArray;
-    var p1 = Object.getPrototypeOf(createArray());
-    var p2 = Object.getPrototypeOf(evalcx("createArray()", sandbox));
-    print(actual = (p1 === p2));
   }
 
-  reportCompare(expect, actual, summary);
+  print('test 1');
+  expect = 'a,c,';
+  actual = '';
+  try
+  {
+    f(createPrint("'a'"), createPrint("'b'"), createPrint("'c'"));
+  }
+  catch(ex)
+  {
+    actual += 'caught ' + ex;
+  }
+  reportCompare(expect, actual, summary + ': 1');
+
+  print('test 2');
+  expect = 'c,caught a';
+  actual = '';
+  try
+  {
+    f(createThrow("'a'"), createPrint("'b'"), createPrint("'c'"));
+  }
+  catch(ex)
+  {
+    actual += 'caught ' + ex;
+  }
+  reportCompare(expect, actual, summary + ': 2');
+
+  print('test 3');
+  expect = 'b,c,';
+  actual = '';
+  try
+  {
+    f(createThrow("null"), createPrint("'b'"), createPrint("'c'"));
+  }
+  catch(ex)
+  {
+    actual += 'caught ' + ex;
+  }
+  reportCompare(expect, actual, summary + ': 3');
+
+  print('test 4');
+  expect = 'a,c,';
+  actual = '';
+  try
+  {
+    f(createPrint("'a'"), createThrow("'b'"), createPrint("'c'"));
+  }
+  catch(ex)
+  {
+    actual += 'caught ' + ex;
+  }
+  reportCompare(expect, actual, summary + ': 4');
+
+  print('test 5');
+  expect = 'c,caught b';
+  actual = '';
+  try
+  {
+    f(createThrow("null"), createThrow("'b'"), createPrint("'c'"));
+  }
+  catch(ex)
+  {
+    actual += 'caught ' + ex;
+  }
+  reportCompare(expect, actual, summary + ': 5');
 
   exitFunc ('test');
 }
