@@ -100,7 +100,6 @@ public:
                               nsIAtom* aAttribute,
                               PRInt32 aModType);
 
-  virtual PRBool GetMouseThrough() const;
   virtual PRBool ComputesOwnOverflowArea() { return PR_FALSE; }
 
 protected:
@@ -115,9 +114,6 @@ protected:
 
  nsLeafBoxFrame(nsIPresShell* aShell, nsStyleContext* aContext);
 
-protected:
-  eMouseThrough mMouseThrough;
-
 private:
 
  void UpdateMouseThrough();
