@@ -2,9 +2,7 @@ var Cc = Components.classes;
 var Ci = Components.interfaces;
 
 function run_test() {
-  var f =
-      Cc["@mozilla.org/file/directory_service;1"].
-      getService(Ci.nsIProperties).get("ComRegF", Ci.nsIFile);
+  var f = do_get_file('test_bug336501.js');
 
   var fis =
       Cc["@mozilla.org/network/file-input-stream;1"].
